//
//  MyOpenAL.m
//
// http://www.gehacktes.net/2009/03/iphone-programming-part-6-multiple-sounds-with-openal/
// Author: hhamm
//

#import "MyOpenAL.h"
#import "MyOpenALSupport.h"

@implementation MyOpenAL

@synthesize bufferStorageArray;
@synthesize soundDictionary;
@synthesize soundBufferDictionary;		// ADDED 2010/3/28 msum

static MyOpenAL *sharedMyOpenAL = nil;

- (void) shutdownMyOpenAL {
	@synchronized(self) {
        if (sharedMyOpenAL != nil) {
			[self dealloc]; // assignment not done here
        }
    }
}

+ (MyOpenAL*)sharedMyOpenAL {
	
    @synchronized(self) {
        if (sharedMyOpenAL == nil) {
            [[self alloc] init]; // assignment not done here
        }
    }
    return sharedMyOpenAL;
}

+ (id)allocWithZone:(NSZone *)zone
{
    @synchronized(self) {
        if (sharedMyOpenAL == nil) {
            sharedMyOpenAL = [super allocWithZone:zone];
            return sharedMyOpenAL;  // assignment and return on first allocation
        }
    }
    return nil; //on subsequent allocation attempts return nil
}

- (id)copyWithZone:(NSZone *)zone
{
    return self;
}

- (id)retain
{
    return self;
}

- (unsigned)retainCount
{
	
	return UINT_MAX;  //denotes an object that cannot be released
}

- (void)release
{
    //do nothing
}

- (id)autorelease
{
    return self;
}

// seek in audio file for the property 'size'
// return the size in bytes
-(UInt32)audioFileSize:(AudioFileID)fileDescriptor
{
	UInt64 outDataSize = 0;
	UInt32 thePropSize = sizeof(UInt64);
	OSStatus result = AudioFileGetProperty(fileDescriptor, kAudioFilePropertyAudioDataByteCount, &thePropSize, &outDataSize);
	if(result != 0) NSLog(@"cannot find file size");
	return (UInt32)outDataSize;
}

// start up openAL
-(bool) initOpenAL
{
	// Initialization
	mDevice = alcOpenDevice(NULL); // select the "preferred device"
	if (mDevice) {
		// use the device to make a context
		mContext=alcCreateContext(mDevice,NULL);
		// set my context to the currently active one
		alcMakeContextCurrent(mContext);
		return true;
	}
	return false;
}

-(id) init {
	if (self = [super init] ) {
		if ([self initOpenAL]) {
			self.bufferStorageArray = [[[NSMutableArray alloc]init]autorelease];
			self.soundDictionary = [[[NSMutableDictionary alloc]init]autorelease];
			self.soundBufferDictionary = [[[NSMutableDictionary alloc]init]autorelease];	// ADDED 2010/3/28 msum			
		}
		return self;
	}
	[self release];
	return nil;
}

-(void) dealloc {
	
	// delete the sources
	for (NSNumber * sourceNumber in [soundDictionary allValues]) {
		NSUInteger sourceID = [sourceNumber unsignedIntegerValue];
		alDeleteSources(1, &sourceID);
	}
	
	self.soundDictionary=nil;
	
	// delete the buffers
	for (NSNumber * bufferNumber in bufferStorageArray) {
		NSUInteger bufferID = [bufferNumber unsignedIntegerValue];
		alDeleteBuffers(1, &bufferID);
	}
	self.bufferStorageArray=nil;
	self.soundBufferDictionary=nil;				// ADDED 2010/3/28 msum
	
	// destroy the context
	alcDestroyContext(mContext);
	// close the device
	alcCloseDevice(mDevice);
	
	[super dealloc];
}

// the main method: grab the sound ID from the library
// and start the source playing
- (void)playSoundWithKey:(NSString*)soundKey
{
	NSNumber * numVal = [soundDictionary objectForKey:soundKey];
	if (numVal == nil) return;
	NSUInteger sourceID = [numVal unsignedIntValue];
	alSourcePlay(sourceID);
}

- (void)stopSoundWithKey:(NSString*)soundKey
{
	NSNumber * numVal = [soundDictionary objectForKey:soundKey];
	if (numVal == nil) return;
	NSUInteger sourceID = [numVal unsignedIntValue];
	alSourceStop(sourceID);
}

-(void) rewindSoundWithKey:(NSString *) soundKey {
	
	NSNumber * numVal = [soundDictionary objectForKey:soundKey];
	if (numVal == nil) return;
	NSUInteger sourceID = [numVal unsignedIntValue];
	alSourceRewind (sourceID);
}

-(bool) isPlayingSoundWithKey:(NSString *) soundKey {
	
	NSNumber * numVal = [soundDictionary objectForKey:soundKey];
	if (numVal == nil) return false;
	NSUInteger sourceID = [numVal unsignedIntValue];
	
	ALenum state;
	
    alGetSourcei(sourceID, AL_SOURCE_STATE, &state);
	
    return (state == AL_PLAYING);
}

-(bool) loadSoundWithKey:(NSString *)_soundKey File:(NSString *)_file Ext:(NSString *) _ext Loop:(bool)loops{
	
	ALvoid * outData;
	ALenum  error = AL_NO_ERROR;
	ALenum  format;
	ALsizei size;
	ALsizei freq;
	
	NSBundle * bundle = [NSBundle mainBundle];
	
	// get some audio data from a wave file
	CFURLRef fileURL = (CFURLRef)[[NSURL fileURLWithPath:[bundle pathForResource:_file ofType:_ext]] retain];
	
	if (!fileURL)
	{
		return false;
	}
	
	outData = MyGetOpenALAudioData(fileURL, &size, &format, &freq);

	CFRelease(fileURL);
	
	if((error = alGetError()) != AL_NO_ERROR) {
		printf("error loading sound: %x\n", error);
		exit(1);
	}
	
	NSUInteger bufferID;
	// grab a buffer ID from openAL
	alGenBuffers(1, &bufferID);
	
	// load the awaiting data blob into the openAL buffer.
	alBufferData(bufferID,format,outData,size,freq); 
	
	// save the buffer so we can release it later
	[bufferStorageArray addObject:[NSNumber numberWithUnsignedInteger:bufferID]];
	
	NSUInteger sourceID;
	// grab a source ID from openAL
	alGenSources(1, &sourceID); 
	
	// attach the buffer to the source
	alSourcei(sourceID, AL_BUFFER, bufferID);
	// set some basic source prefs
	alSourcef(sourceID, AL_PITCH, 1.0f);
	alSourcef(sourceID, AL_GAIN, 1.0f);
	if (loops) alSourcei(sourceID, AL_LOOPING, AL_TRUE);
	
	// store this for future use
	[soundDictionary setObject:[NSNumber numberWithUnsignedInt:sourceID] forKey:_soundKey];

	[soundBufferDictionary setObject:[NSNumber numberWithUnsignedInt:bufferID] forKey:_soundKey];	// ADDED 2010/3/28 msum

	// clean up the buffer
	if (outData)
	{
		free(outData);
		outData = NULL;
	}
	
	return true;
	
}

// ADDED
- (bool) loadedSoundWithKey:(NSString *)_soundKey{
	NSNumber * numVal = [soundDictionary objectForKey:_soundKey];
	if (numVal == nil) return false;
	return true;
}
- (void) releaseSoundWithKey:(NSString *)_soundKey{
	NSNumber * numVal = [soundDictionary objectForKey:_soundKey];
	if (numVal != nil){
		NSUInteger sourceID = [numVal unsignedIntegerValue];
		alDeleteSources(1, &sourceID);
		[soundDictionary removeObjectForKey:_soundKey];
	}
	numVal = [soundBufferDictionary objectForKey:_soundKey];
	if (numVal != nil){
		NSUInteger bufferID = [numVal unsignedIntegerValue];
		alDeleteBuffers(1, &bufferID);
		[soundBufferDictionary removeObjectForKey:_soundKey];
	}
}

@end